<?php 
namespace noKvm;

require(ROOTDIR . "/modules/servers/noKvm/vendor/autoload.php");

class noKvm
{
    protected $url = NULL;
    protected $accesshash = NULL;
    protected $serverport = NULL;
    protected $extData = NULL;
    protected $masterID = NULL;
    public $result = "";
    public $rawResult = "";
    public $ipaddress = NULL;
    protected $params = array(  );
    protected $configOption = NULL;
    protected $serviceid = NULL;
    protected $pid = NULL;

    public function __construct($params, $debug = false)
    {
        if( $debug === true ) 
        {
            ini_set("display_errors", 1);
            ini_set("display_startup_errors", 1);
            error_reporting(E_ALL);
        }

        if( is_array($params) ) 
        {
            $this->params = $params;
        }

        $this->masterID = $this->getParam("serverid");
        $row = \Illuminate\Database\Capsule\Manager::table("tblservers")->where("id", $this->masterID)->first();
        $this->accesshash = $row->accesshash;
        $this->serverport = $row->serverport;
        $this->url = "http://" . $row->ipaddress . "/api/";
        $this->configOption = $this->getParam("configoptions");
        $this->ipaddress = $row->ipaddress;
        $this->serviceid = $this->getParam("serviceid");
        $this->pid = $this->getParam("pid");
    }

    public function getParam($name)
    {
        if( isset($this->params[$name]) ) 
        {
            return $this->params[$name];
        }

        return "";
    }

    public function getExtData($name)
    {
        if( isset($this->extData[$name]) ) 
        {
            return $this->extData[$name];
        }

        return "";
    }

    private function createNonceStr($length = 32)
    {
        $chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
        $str = "";
        for( $i = 0; $i < $length; $i++ ) 
        {
            $str .= substr($chars, mt_rand(0, strlen($chars) - 1), 1);
        }
        return $str;
    }

    private function arithmetic($timeStamp, $randomStr, $token)
    {
        $arr["timeStamp"] = $timeStamp;
        $arr["randomStr"] = $randomStr;
        $arr["token"] = $token;
        sort($arr, SORT_STRING);
        $str = implode($arr);
        $signature = md5($str);
        $signature = strtoupper($signature);
        return $signature;
    }

    public function signature($token)
    {
        $timeStamp = time();
        $randomStr = $this->createNonceStr();
        $signature = $this->arithmetic($timeStamp, $randomStr, $token);
        return array( "time" => $timeStamp, "random" => $randomStr, "signature" => $signature );
    }

    public function TestConnection($ipaddr, $token)
    {
        $signature = $this->signature($token);
        $url = "http://" . $ipaddr . "/api/area";
        $signature = "?time=" . $signature["time"] . "&random=" . $signature["random"] . "&signature=" . $signature["signature"];
        $client = new \GuzzleHttp\Client(array( "verify" => false ));
        $result = $client->get($url . $signature);
        $result = json_decode($result->getBody());
        return $result;
    }

    public function noVNC($vmID)
    {
        $signature = $this->signature($this->accesshash);
        return $this->url . "virtual_link_vnc/" . $vmID . "?time=" . $signature["time"] . "&random=" . $signature["random"] . "&signature=" . $signature["signature"];
    }

    public function apiCall($action = "POST", $faction, $postVars = array(  ))
    {
        $this->jsonResult = "";
        $this->result = "";
        try
        {
            $client = new \GuzzleHttp\Client(array( "base_uri" => $this->url, "verify" => false ));
            $signature = $this->signature($this->accesshash);
            if( $action == "GET" ) 
            {
                $signature = "?time=" . $signature["time"] . "&random=" . $signature["random"] . "&signature=" . $signature["signature"];
                $result = $client->get($faction . $signature);
            }
            else
            {
                $postVars["time"] = $signature["time"];
                $postVars["random"] = $signature["random"];
                $postVars["signature"] = $signature["signature"];
                $signature = "";
                $postArry = array( "form_params" => $postVars );
                $result = $client->request($action, $faction, $postArry);
            }

            $result->result = json_decode($result->getBody());
        }
        catch( RequestException $e ) 
        {
            return $e->getRequest();
        }
        return $result;
    }

    public function serverExtra($mServer = array(  ))
    {
        if( !isset($mServer["master"]) ) 
        {
            return array(  );
        }

        $results = array(  );
        $masterPart = explode("-", $mServer["master"]);
        $row = \Illuminate\Database\Capsule\Manager::table("tblservers")->where("id", $masterPart[0])->first();
        if( $row ) 
        {
            $data = htmlspecialchars_decode($row->accesshash);
            $results = $this->sortReturn($data);
        }

        if( !is_array($results) ) 
        {
            return array(  );
        }

        return $results;
    }

    public function passwordGen($length = 10, $chars = "abcdefghijklmnopqrstuvwxyzABCEFGHIJKLMNOPQRSTUVWXYZ1234567890")
    {
        $chars_length = strlen($chars) - 1;
        $string = $chars[rand(0, $chars_length)];
        $i = 1;
        while( $i < $length ) 
        {
            $r = $chars[rand(0, $chars_length)];
            if( $r != $string[$i - 1] ) 
            {
                $string .= $r;
            }

            $i = strlen($string);
        }
        return $string;
    }

    public function getSystem()
    {
        $ossplit = explode("|", $this->getParam("configoption6"));
        $buildOperatingSystem = $ossplit[0];
        return $buildOperatingSystem;
    }

    public function getBuildOS()
    {
        $configType = "Operating System";
        if( !isset($this->configOption[$configType]) ) 
        {
            $buildOperatingSystem = explode("|", $this->getParam("configoption3"));
            $buildOperatingSystem = $buildOperatingSystem[0];
        }
        else
        {
            $ossplit = explode("|", $this->configOption[$configType]);
            $buildOperatingSystem = $ossplit[0];
        }

        return $buildOperatingSystem;
    }

    public function getBuildGroup()
    {
        $configTypeG = "Location";
        if( !isset($this->configOption[$configTypeG]) ) 
        {
            $splitGroup = explode("|", $this->getParam("configoption1"));
            $configOptionNodeGroup = $splitGroup[0];
            $buildGroup = $configOptionNodeGroup;
        }
        else
        {
            $groupsplit = explode("|", $this->configOption[$configTypeG]);
            $buildGroup = $groupsplit[0];
        }

        return $buildGroup;
    }

    public function getPlan($data)
    {
        $Plan = $this->apiCall("GET", "plan/" . $data)->result->data;
        return $Plan;
    }

    public function getCcpu()
    {
        $custom_config_cpu = "CPU";
        if( !isset($this->configOption[$custom_config_cpu]) ) 
        {
            $splitCPU = explode("|", $this->getParam("configoption2"));
            $result = $this->getPlan($splitCPU[0]);
            $ccpu = $result->core;
        }
        else
        {
            $cpusplit = explode("|", $this->configOption[$custom_config_cpu]);
            $ccpu = $cpusplit[0];
        }

        return $ccpu;
    }

    public function getCcpuMode()
    {
        $splitMode = explode("|", $this->getParam("configoption5"));
        $ccpumode = $splitMode[0];
        return $ccpumode;
    }

    public function getCmem()
    {
        $custom_config_memory = "Memory";
        if( !isset($this->configOption[$custom_config_memory]) ) 
        {
            $splitRAM = explode("|", $this->getParam("configoption2"));
            $result = $this->getPlan($splitRAM[0]);
            $cmem = $result->memory;
        }
        else
        {
            $memsplit = explode("|", $this->configOption[$custom_config_memory]);
            $cmem = $memsplit[0];
        }

        return $cmem;
    }

    public function getCdisk()
    {
        $custom_config_diskspace = "Disk Space";
        if( !isset($this->configOption[$custom_config_diskspace]) ) 
        {
            $splitDISK = explode("|", $this->getParam("configoption2"));
            $result = $this->getPlan($splitDISK[0]);
            $cdisk = $result->data_disk_size;
        }
        else
        {
            $disksplit = explode("|", $this->configOption[$custom_config_diskspace]);
            $cdisk = $disksplit[0];
        }

        return $cdisk;
    }

    public function getCnspeed()
    {
        $custom_config_nspeed = "Network Speed";
        if( !isset($this->configOption[$custom_config_nspeed]) ) 
        {
            $splitNET = explode("|", $this->getParam("configoption2"));
            $result = $this->getPlan($splitNET[0]);
            $cextraspeed = $result->net_out;
        }
        else
        {
            $extraspeedsplit = explode("|", $this->configOption[$custom_config_nspeed]);
            $cextraspeed = $extraspeedsplit[0];
        }

        return $cextraspeed;
    }

    public function getSnapshot()
    {
        $custom_config_nspeed = "Snapshot";
        if( !isset($this->configOption[$custom_config_nspeed]) ) 
        {
            $splitSnapshot = explode("|", $this->getParam("configoption2"));
            $result = $this->getPlan($splitSnapshot[0]);
            $cextraspeed = $result->snapshoot;
        }
        else
        {
            $extraspeedsplit = explode("|", $this->configOption[$custom_config_nspeed]);
            $cextraspeed = $extraspeedsplit[0];
        }

        return $cextraspeed;
    }

    public function getBackups()
    {
        $custom_config_nspeed = "Backups";
        if( !isset($this->configOption[$custom_config_nspeed]) ) 
        {
            $splitBackups = explode("|", $this->getParam("configoption2"));
            $result = $this->getPlan($splitBackups[0]);
            $cextraspeed = $result->backups;
        }
        else
        {
            $extraspeedsplit = explode("|", $this->configOption[$custom_config_nspeed]);
            $cextraspeed = $extraspeedsplit[0];
        }

        return $cextraspeed;
    }

    public function getCbandwidth()
    {
        $custom_config_bandwidth = "Bandwidth";
        if( !isset($this->configOption[$custom_config_bandwidth]) ) 
        {
            $cbandwidth = "";
        }
        else
        {
            $bandwidthsplit = explode("|", $this->configOption[$custom_config_bandwidth]);
            $cbandwidth = $bandwidthsplit[0];
        }

        return $cbandwidth;
    }

    public function getCextraip()
    {
        $custom_config_extraip = "Extra IP Address";
        if( !isset($this->configOption[$custom_config_extraip]) ) 
        {
            $cextraip = "";
        }
        else
        {
            $extraipsplit = explode("|", $this->configOption[$custom_config_extraip]);
            $cextraip = $extraipsplit[0];
        }

        return $cextraip;
    }

    public function getHostname()
    {
        $newHostname = "ECS-" . MD5(time());
        return $newHostname;
    }

    public function getNewDataPassword()
    {
        if( $this->getParam("password") == "auto-generated" || $this->getParam("password") == "" ) 
        {
            $this->params["password"] = $this->passwordGen();
            $passwordenc = encrypt($this->params["password"]);
            \Illuminate\Database\Capsule\Manager::table("tblhosting")->where("id", $this->serviceid)->update(array( "password" => $passwordenc ));
        }

        $newData = \Illuminate\Database\Capsule\Manager::table("tblhosting")->where("id", $this->serviceid)->first();
        $newDataPassword = decrypt($newData->password);
        return $newDataPassword;
    }

    public function removeipTerminatedProduct()
    {
        if( $this->getExtData("removeip-terminated-product") != "no" ) 
        {
            \Illuminate\Database\Capsule\Manager::table("tblhosting")->where("id", $this->serviceid)->update(array( "dedicatedip" => "", "assignedips" => "" ));
        }

    }

    public function removesnapshotTerminatedProduct()
    {
        $customFields = $this->getParam("customfields");
        $data = array( "id" => $customField["vserverid"] );
        $result = $this->apiCall("panel_delete_snapshot", $data)->result;
    }

    public function removebackupsTerminatedProduct()
    {
        $customFields = $this->getParam("customfields");
        $data = array( "id" => $customField["vserverid"] );
        $result = $this->apiCall("panel_delete_fullbackup", $data)->result;
    }

    public function removevserveridTerminatedProduct()
    {
        if( $this->getExtData("removevserverid-terminated-product") != "no" ) 
        {
            $this->setCustomfieldsValue("vserverid", "");
        }

    }

    public function setCustomfieldsValue($field, $value)
    {
        $value = (string) $value;
        $res = \Illuminate\Database\Capsule\Manager::table("tblcustomfields")->where("relid", $this->pid)->where("fieldname", $field)->first();
        if( $res ) 
        {
            $fieldValue = \Illuminate\Database\Capsule\Manager::table("tblcustomfieldsvalues")->where("relid", $this->serviceid)->where("fieldid", $res->id)->first();
            if( $fieldValue ) 
            {
                if( $fieldValue->value !== $value ) 
                {
                    \Illuminate\Database\Capsule\Manager::table("tblcustomfieldsvalues")->where("relid", $this->serviceid)->where("fieldid", $res->id)->update(array( "value" => $value ));
                }

            }
            else
            {
                \Illuminate\Database\Capsule\Manager::table("tblcustomfieldsvalues")->insert(array( "relid" => $this->serviceid, "fieldid" => $res->id, "value" => $value ));
            }

        }

    }

    public static function validateRootPassword($newRootPassword)
    {
        $is_valid = false;
        if( 5 < strlen($newRootPassword) && preg_match("/^[a-zA-Z0-9\\-_]{0,50}\$/i", $newRootPassword) ) 
        {
            $is_valid = true;
        }

        return $is_valid;
    }

    public static function validateVNCPassword($newVNCPassword)
    {
        $is_valid = false;
        if( 5 < strlen($newVNCPassword) && preg_match("/^[a-zA-Z0-9\\-_]{0,50}\$/i", $newVNCPassword) ) 
        {
            $is_valid = true;
        }

        return $is_valid;
    }

    public static function getParamsFromServiceID($servid, $uid = NULL)
    {
        $ownerRow = \Illuminate\Database\Capsule\Manager::table("tblhosting")->where("id", $servid)->where("userid", $uid)->first();
        if( !$ownerRow ) 
        {
            return false;
        }

        if( !is_null($uid) && $uid != $ownerRow->userid ) 
        {
            return false;
        }

        $vserverFieldRow = \Illuminate\Database\Capsule\Manager::table("tblcustomfields")->where("relid", $ownerRow->packageid)->where("fieldname", "vserverid")->first();
        if( !$vserverFieldRow ) 
        {
            return false;
        }

        $vserverValueRow = \Illuminate\Database\Capsule\Manager::table("tblcustomfieldsvalues")->where("fieldid", $vserverFieldRow->id)->where("relid", $ownerRow->id)->first();
        if( !$vserverValueRow ) 
        {
            return false;
        }

        if( !is_numeric($vserverValueRow->value) ) 
        {
            return false;
        }

        $serverid = $ownerRow->server;
        if( $serverid == "0" || $serverid == "" ) 
        {
            $productRow = \Illuminate\Database\Capsule\Manager::table("tblproducts")->where("id", $ownerRow->packageid)->first();
            $masterPart = explode("|", $productRow->configoption1);
            $serverid = $masterPart[0];
        }

        return array( "serverid" => $serverid, "vserver" => $vserverValueRow->value, "serviceid" => $vserverValueRow->relid, "pid" => $vserverFieldRow->relid );
    }

    public static function getParamsFromVserviceID($vserverid, $uid)
    {
        foreach( \Illuminate\Database\Capsule\Manager::table("tblhosting")->where("userid", $uid)->get() as $hosting ) 
        {
            $vserverFieldRow = \Illuminate\Database\Capsule\Manager::table("tblcustomfields")->where("relid", $hosting->packageid)->where("fieldname", "vserverid")->first();
            if( !$vserverFieldRow ) 
            {
                continue;
            }

            $vserverValueRow = \Illuminate\Database\Capsule\Manager::table("tblcustomfieldsvalues")->where("fieldid", $vserverFieldRow->id)->where("relid", $hosting->id)->first();
            if( !$vserverValueRow ) 
            {
                continue;
            }

            if( $vserverid == $vserverValueRow->value ) 
            {
                $serverid = $hosting->server;
                return array( "serverid" => $serverid, "vserver" => $vserverid, "serviceid" => $vserverValueRow->relid, "pid" => $vserverFieldRow->relid );
            }

        }
        return false;
    }

    public function bwFormat($size)
    {
        $bytes = array( " KB", " MB", " GB", " TB" );
        $resVal = "";
        foreach( $bytes as $val ) 
        {
            $resVal = $val;
            if( 1024 <= $size ) 
            {
                $size = $size / 1024;
            }
            else
            {
                break;
            }

        }
        return round($size, 1) . $resVal;
    }

    public function formatSize($size)
    {
        $sizes = array( " Bytes", " KB", " MB", " GB", " TB" );
        if( empty($size) || $size == "0.0" || $size == "0.00" || $size == "0" ) 
        {
            return "n/a";
        }

        return round($size / pow(1024, $i = floor(log($size, 1024))), 2) . $sizes[$i];
    }

    public function setHostname($newhostname)
    {
        if( !empty($this->serviceid) ) 
        {
            \Illuminate\Database\Capsule\Manager::table("tblhosting")->where("id", $this->serviceid)->update(array( "domain" => $newhostname ));
        }

    }

    public static function loadLang($lang = NULL)
    {
        global $_LANG;
        global $CONFIG;
        $langDir = __DIR__ . "/../lang/";
        $availableLangsFullPath = glob($langDir . "*.php");
        $availableLangs = array(  );
        foreach( $availableLangsFullPath as $availableLang ) 
        {
            $availableLangs[] = strtolower(basename($availableLang));
        }
        if( empty($lang) ) 
        {
            if( isset($_SESSION["Language"]) ) 
            {
                $language = $_SESSION["Language"];
            }
            else
            {
                if( isset($_SESSION["adminlang"]) ) 
                {
                    $language = $_SESSION["adminlang"];
                }
                else
                {
                    $language = $CONFIG["Language"];
                }

            }

        }
        else
        {
            $language = $lang;
        }

        $language = strtolower($language) . ".php";
        if( !in_array($language, $availableLangs) ) 
        {
            $language = "english.php";
        }

        require_once($langDir . $language);
    }

}
?>